import tokenize

last_line_number = 0

def init_parser():
    last_line_number = 0

def get_last_line():
    return last_line_number

class success_parser(object):
     def __init__(self, return_val):
        self.return_val = return_val
     def parse(self, toks_pair):
        return self.return_val;

class literal_parser(object):
    def __init__(self, expected_tok, expected_val, func=None):
        self.expected_tok = expected_tok
        self.expected_val = expected_val
        self.create_func = func
    def parse(self, toks_pair):
        (toks, loc) = toks_pair
        if (toks[loc][0] != self.expected_tok):
            return None
        if (self.expected_val):
            if  (toks[loc][1] == self.expected_val):
                global last_line_number
                last_line_number = toks[loc][2]
                toks_pair[1] += 1
                if (self.create_func):
                    return self.create_func(self.expected_val)
                else:
                    return self.expected_val
            else:
                return None
        else:
            toks_pair[1] += 1
            if (self.create_func):
                return self.create_func(toks[loc][1])
            else:
                return toks[loc][1]

class sequence_parser(object):
    def __init__(self, child_parsers, func=None):
        self.children = child_parsers
        self.create_func = func
    def parse(self, toks_pair):
        (toks, original_loc) = toks_pair
        results = []
        for parser in self.children:
            output = parser.parse(toks_pair)
            if (output != None):
                results.append( output )
            else:
                toks_pair[1] = original_loc
                return None
        if (self.create_func):
            return self.create_func(results)
        else:
            return results

class disjunction_parser(object):
    def __init__(self, child_parsers):
        self.children = child_parsers
    def parse(self, toks_pair):
        for p in self.children:
            toks_copy = list(toks_pair)
            output = p.parse(toks_copy)
            if (output != None):
                toks_pair[1] = toks_copy[1]
                return output
        return None

class kleene_star_parser(object):
    def __init__(self, child, func=None):
        self.child_parser = child
        self.create_func = func
    def parse(self, toks_pair):
        success = True
        results = []
        while (success):
            parser_output = self.child_parser.parse(toks_pair)
            if (parser_output != None):
                results.append( parser_output )
            else:
                success = False
        if (self.create_func):
            return self.create_func( results )
        else:
            return results

class number_parser(literal_parser):
    def __init__(self, func=None):
        super(number_parser, self).__init__(tokenize.NUMBER, None, func)

class string_parser(literal_parser):
    def __init__(self, func=None):
        super(string_parser, self).__init__(tokenize.STRING, None, func)

class keyword_parser(literal_parser):
    def __init__(self, keyword, func=None):
        super(keyword_parser, self).__init__(tokenize.NAME, keyword, func )

class op_parser(literal_parser):
    def __init__(self, op, func=None):
        super(op_parser, self).__init__(tokenize.OP, op, func )

class newline_parser(literal_parser):
    def __init__(self):
        super(newline_parser, self).__init__(tokenize.NEWLINE, None, None)

class indent_parser(literal_parser):
    def __init__(self):
        super(indent_parser, self).__init__(tokenize.INDENT, None, None)

class end_parser(literal_parser):
    def __init__(self):
        super(end_parser, self).__init__(tokenize.ENDMARKER, None, None)

class dedent_parser(literal_parser):
    def __init__(self):
        super(dedent_parser, self).__init__(tokenize.DEDENT, None, None)

class identifier_parser(literal_parser):
    def __init__(self, func=None):
        super(identifier_parser,self).__init__(tokenize.NAME, None, func )

