from optparse import OptionParser
import tokenize
from StringIO import StringIO
from parser_objs import init_parser
from parser_objs import get_last_line
import snurtle_parser
import time

#def extract_tokens(inp):
#    toks = [(tok, val, start_char[0]) for (tok, val, start_char, _, _) in tokenize.generate_tokens(inp.readline) if
#            (tok != tokenize.NL and tok != tokenize.COMMENT)]
#    return toks


def main():
    parser = OptionParser(usage="usage: %prog [options]",
                          version="%prog 1.0")

    parser.add_option("-o", "--output_file",
                      action="store",
                      default="output.svg",
                      help="output file")

    parser.add_option("-s", "--start_sub",
                      action="store",
                      default="main",
                      help="substitution definition to start with")

    parser.add_option("-c", "--coordinates",
                      action="store",
                      default="400,400",
                      help="common separated x,y coords of starting point")

    parser.add_option("-d", "--dimensions",
                      action="store",
                      default="800,800",
                      help="common separated with and height")

    parser.add_option("-k", "--scale",
                      action="store",
                      type="float",
                      default=100.0,
                      help="common separated with and height")

    parser.add_option("-w", "--html",
                      action="store_true",
                      default="",
                      help="output html rather than SVG")

    parser.add_option("-m", "--max_depth",
                      action="store",
                      type="int",
                      default=3,
                      help="Maximum number of stack levels before terminating recursion")

    try:
        (options, args) = parser.parse_args()

        input_file = ""
        if len(args) == 1:
            input_file = args[0]
        else:
            raise Exception("Specify an input file.")

        toks = extract_tokens( input_file )

        snurt_parser = snurtle_parser.script_parser()
        tokstream = [toks, 0]

        init_parser()
        with Timer() as t:
            print "Parsing %s ..." % input_file
            tile_set = snurt_parser.parse(tokstream);

            if not tile_set:
                raise Exception("Error parsing %s around line %d" % (input_file, get_last_line()))

            (width, height) = map(float, options.dimensions.split(","))
            (x, y) = map(float, options.coordinates.split(","))

            print "Processing  ..."
            output = tile_set.generate(options.start_sub, (x, y), options.max_depth, options.scale)

            print "Writing %s ..." % options.output_file
            text_file = open(options.output_file, "w")
            text_file.write(output.to_svg(options.html, width, height))
            text_file.close()
        print "Complete. (running time = %.04f sec.)" % t.interval
    except Exception as e:
        print "\nError: %s" % e
        return

def extract_tokens(filename):
    # read the entire file into a string so we can ensure that it ends with an EOL
    inp = open(filename, 'r')
    script = inp.read()
    inp.close()

    # ensure there is an EOF ... otherwise tokenization will screw up relative to the expected grammar...
    # and it's easier to fix this here than during parsing.
    if (script[-1] != '\n'):
        script = script + '\n'

    toks = [(tok, val, start_char[0]) for (tok, val, start_char, _, _) in tokenize.generate_tokens(StringIO(script).readline) if
            (tok != tokenize.NL and tok != tokenize.COMMENT)]
    return toks

class Timer:
    def __enter__(self):
        self.start = time.clock()
        return self

    def __exit__(self, *args):
        self.end = time.clock()
        self.interval = self.end - self.start


if __name__ == '__main__':
    main()

