import tokenize
from parser_objs import *
from snurtle_objs import *

#------------------------------------------------------------------------------#
class factor_parser(disjunction_parser):
    def __init__(self, expr_parser):
        def negative_factor_parser():
            def create_negation(x):
                return expr_negate(x[1])
            return sequence_parser([op_parser("-",None), self], create_negation)

        def paren_expr_parser():
            return sequence_parser(
                [op_parser("(",None), expr_parser, op_parser(")",None)],
                lambda o: o[1]
            )

        super(factor_parser,self).__init__([
            negative_factor_parser(),
            paren_expr_parser(),
            number_parser(lambda n: expr_num(n)),
            identifier_parser(lambda v: expr_var(v)),
            literal_parser(tokenize.STRING, None, None )
        ])

class expr_parser_aux(sequence_parser):
    def __init__(self, aux, op_strings):
        def end_of_term_parser():
            op_parsers = [ op_parser(op_str,None) for op_str in op_strings ]
            return kleene_star_parser(
                sequence_parser(
                    [  disjunction_parser(op_parsers), aux  ],
                    None
                ),
                None
            )
        def make_term(pair):
            head = pair[0]
            tail = pair[1]
            child_parsers = [head]
            ops = []
            for (op, child_parser) in tail:
                ops.append(op)
                child_parsers.append(child_parser)
            return expr(child_parsers, ops) if ops else head
        super(expr_parser_aux,self).__init__([aux, end_of_term_parser()], make_term)

class term_parser(expr_parser_aux):
    def __init__(self, expr_parser):
        super(term_parser,self).__init__(factor_parser(expr_parser), ['*','/'])

class expr_parser(expr_parser_aux):
    def __init__(self):
        super(expr_parser,self).__init__(term_parser(self), ['+','-'])

#------------------------------------------------------------------------------#

class list_parser(disjunction_parser):
    def __init__(self, child_parser, delimiter_parser):
        def create_func(parse_results):
            return [parse_results[0]] + [ x for (delimiter, x) in parse_results[1] ]
        super(list_parser, self).__init__( [
                sequence_parser( [
                        child_parser,
                        kleene_star_parser(
                            sequence_parser([delimiter_parser, child_parser], None),
                            None
                        )
                    ],
                    create_func
                ),
                success_parser([])
            ]
        )

#class pass_parser(keyword_parser):
#    def __init__(self):
#        super(pass_parser,self).__init__( "pass", lambda x : pass_stmt() )

class pass_parser(sequence_parser):
    def __init__(self):
        super(pass_parser,self).__init__( [keyword_parser("pass"), newline_parser()], lambda x : pass_stmt()  )

class statement_parser(sequence_parser):
    def __init__(self, name_parser, args_parser, create_func):
        super(statement_parser,self).__init__(
            # <keyword>( args )\n
            [name_parser, op_parser("("), args_parser, op_parser(")"), newline_parser()],
            create_func
        )

class forward_stmt_parser(statement_parser):
    def __init__(self) :
        super(forward_stmt_parser,self).__init__(
            keyword_parser("forward"),
            expr_parser(),
            lambda x: forward_stmt(x[2])
        )

class turn_stmt_parser(statement_parser):
    def __init__(self) :
        super(turn_stmt_parser,self).__init__(
            keyword_parser("turn"),
            expr_parser(),
            lambda x: turn_stmt(x[2])
        )

class call_subroutine_parser(statement_parser):
    def __init__(self) :
        super(call_subroutine_parser,self).__init__(
            identifier_parser(None),
            list_parser(expr_parser(),op_parser(",")),
            lambda x: subroutine_stmt(x[0], x[2])
        )

class multiline_stmt_parser(sequence_parser):
    def __init__(self, stmt_list_parser, name_parser, args_parser, create_func):
        super(multiline_stmt_parser, self).__init__( [
                name_parser,             #0
                op_parser("("),          #1
                args_parser,             #2
                op_parser(")"),          #3
                op_parser(":"),          #4
                newline_parser(),        #5
                stmt_list_parser,        #6
            ],
            create_func
        )

class fill_stmt_parser(multiline_stmt_parser):
    def __init__(self, stmt_list_parser):
        def create_func(x):
            return fill_stmt(x[2], x[6])
        super(fill_stmt_parser, self).__init__(
            stmt_list_parser,
            keyword_parser("fill"),
            string_parser(),
            create_func
        )

class stroke_stmt_parser(multiline_stmt_parser):
    def __init__(self, stmt_list_parser):
        def create_func(x):
            (color, _ , thickness) = x[2]
            return stroke_stmt(color, thickness, x[6])
        super(stroke_stmt_parser, self).__init__(
            stmt_list_parser,
            keyword_parser("stroke"),
            sequence_parser([string_parser(), op_parser(","), expr_parser()]),
            create_func
        )

class poly_stmt_parser(multiline_stmt_parser):
    def __init__(self, stmt_list_parser):
        def create_func(x):
            (fill_color, _, stroke_color , _, thickness) = x[2]
            return poly_stmt(fill_color, stroke_color, thickness, x[6])
        super(poly_stmt_parser, self).__init__(
            stmt_list_parser,
            keyword_parser("poly"),
            sequence_parser([string_parser(),op_parser(","), string_parser(), op_parser(","), expr_parser()]),
            create_func
        )

class statement_list_parser(sequence_parser):
    def __init__(self):
        super(statement_list_parser,self).__init__([
                indent_parser(),
                kleene_star_parser(disjunction_parser([
                    forward_stmt_parser(),
                    turn_stmt_parser(),
                    call_subroutine_parser(),
                    fill_stmt_parser(self),
                    stroke_stmt_parser(self),
                    branch_stmt_parser(self),
                    flip_stmt_parser(self),
                    pass_parser(),
                    poly_stmt_parser(self)
                ])),
                dedent_parser()
            ],
            lambda x : x[1]
        )

class branch_stmt_parser(sequence_parser):
    def __init__(self, stmt_list_parser):
        super(branch_stmt_parser, self).__init__( [
                keyword_parser("branch"),  #0
                op_parser(":"),            #1
                newline_parser(),          #2
                stmt_list_parser,          #3
            ],
            lambda x : branch_stmt( x[3] )
        )

class flip_stmt_parser(sequence_parser):
    def __init__(self, stmt_list_parser):
        super(flip_stmt_parser, self).__init__( [
                keyword_parser("flip"),  #0
                op_parser(":"),            #1
                newline_parser(),          #2
                stmt_list_parser,          #3
            ],
            lambda x : flip_stmt( x[3] )
        )

class tile_def_parser(sequence_parser):
    def __init__(self):
        def create_func(x) :
            return substitution_def(x[1], x[3], x[11], x[15])
        super(tile_def_parser, self).__init__([
            keyword_parser("sub"),                            #0
            identifier_parser(),                              #1
            op_parser("("),                                   #2
            list_parser(identifier_parser(), op_parser(",")), #3
            op_parser(")"),                                   #4
            op_parser(":"),                                   #5
            newline_parser(),                                 #6
            indent_parser(),                                  #7
            keyword_parser("terminal"),                       #8
            op_parser(":"),                                   #9
            newline_parser(),                                 #10
            statement_list_parser(),                          #11
            keyword_parser("nonterminal"),                    #12
            op_parser(":"),                                   #13
            newline_parser(),                                 #14
            statement_list_parser(),                          #15
            dedent_parser(),                                  #16
        ],
        create_func
        )

class script_parser(sequence_parser):
    def __init__(self):
        def create_script(x):
            return snurtle_script( x[0] )
        super(script_parser,self).__init__([
                kleene_star_parser( tile_def_parser() ),
                end_parser()
            ],
            create_script
        )